function locke_06(modelname, tmax, mode)

if nargin < 3 
    mode = 'run';
end

switch mode
    case 'run'
        try
            run(modelname, tmax);
        catch
            [msgstr msgid] = lasterr;
            ReturnMsg(msgstr);
        end
    case 'final'
        try
            final(modelname);
        catch
            [msgstr msgid] = lasterr;
            ReturnMsg(msgstr);
        end
end

%----------------------------------------------------------------------
function run(modelname, tmax)

%needs num colours, hastemp, hasld, hasvern
global PARAM_VALS;
global LD_DEF;
global TEMP_DEF;
global VERN_VALS;
global PETURBED_PARAMS;

global RESULTS;
global OUTPUT;
global TIME_POINTS;
global LD_CYCLE;
global TEMP_CYCLE;
global PARAMETERS;

global OPREQUIRED;
global HAS_LD;
global HAS_TEMP;
global HAS_VERN;

global PARAM_PET;
global STATE_PET;

global ENVFILE;
ENVFILE = [];

%read settings
[Settings, SettingVals] = textread('Settings.txt','%[^=]%*c%[^\n]');
%find LD
for j = 1:size(Settings, 1); %search names for a match with state name
    %when found, get corresponding value
    if strcmp(Settings(j), 'maxStep') ~= 0
        maxStep = str2double(SettingVals(j));
    elseif strcmp(Settings(j), 'relt') ~= 0
        relt = str2double(SettingVals(j));
    elseif strcmp(Settings(j), 'abst') ~= 0
        abst = str2double(SettingVals(j));     
    elseif strcmp(Settings(j), 'refine') ~= 0
        refine = str2double(SettingVals(j));
    elseif strcmp(Settings(j), 'solver') ~= 0
        ODESolver = SettingVals(j);
    elseif strcmp(Settings(j), 'NMax') ~= 0 %bvs only
        NMax = SettingVals(j);
    elseif strcmp(Settings(j), 'envFile') ~= 0 
        ENVFILE = SettingVals(j);
    end
end
delete('Settings.txt')
%command line params will be string in the compiled version
if ~isnumeric(tmax)
    tmax = str2double(tmax);
end

%estimate of the number of integration steps per day, used to pre-allocate
%memory
PointsPerDay = 50;
%read which outputs the interface has requested into a col vector
OPREQUIRED = textread(strcat(modelname, '.ops'), '%d');

%get parameter values
fid = fopen('Params.txt', 'rt');
PARAM_VALS = fscanf(fid, '%f');
status = fclose(fid);
delete('Params.txt');
fid = fopen('States.txt', 'rt');
State_Vals = fscanf(fid, '%f');
status = fclose(fid);
delete('States.txt');

if exist('LD.txt', 'file') == 2
    fid = fopen('LD.txt', 'rt');
    LD_DEF = [];
    while ~feof(fid)
        str = fgets(fid);
        if str ~= -1
            row = sscanf(str, '%f'); 
            LD_DEF = [LD_DEF; row'];
        end
    end
    status = fclose(fid);
    delete('LD.txt');
    HAS_LD = size(LD_DEF,2) - 1;
else
    HAS_LD = 0;
end
if exist('Temp.txt', 'file') == 2
    fid = fopen('Temp.txt', 'rt');
    TEMP_DEF = [];
    while ~feof(fid)
        row = fscanf(fid, '%f',2);
        TEMP_DEF = [TEMP_DEF; row'];
    end
    status = fclose(fid);
    delete('Temp.txt');
    HAS_TEMP = 1;
else
    HAS_TEMP = 0;
end
if exist('Vern.txt', 'file') == 2
    fid = fopen('Vern.txt', 'rt');
    VERN_VALS = cell(0);
    while ~feof(fid)
        str = fgets(fid);
        VERN_VALS{end + 1} = str;
    end
    status = fclose(fid);
    delete('Vern.txt');
    HAS_VERN = 1;
else
    HAS_VERN = 0;
end

%look for any parameter peturbations required
PARAM_PET = [];
PETURBED_PARAMS = [];
if exist(strcat(modelname, '.pmpt'),'file') == 2
    %file is a matrix with one row for each parameter to be perturbed.
    %col 1 is param number, col2 is time on, col3 is time off
    %col 4 is size of peturbation, col5 is what this means. a - absolute
    %size, f - a factor to multiply existing value by, s = sum with
    %exisitng vlaue
    [param_no t_on t_off pet_size pet_type] = textread(strcat(modelname, '.pmpt'),'%d %f %f %f %c');
    for i = 1:size(param_no,1)
        PARAM_PET(i).param = param_no(i);
        PETURBED_PARAMS = [PETURBED_PARAMS; param_no(i)];
        PARAM_PET(i).on = t_on(i);
        PARAM_PET(i).off = t_off(i);
        PARAM_PET(i).size = pet_size(i);
        PARAM_PET(i).type = pet_type(i);
    end
    %remove duplicates from list
    PETURBED_PARAMS = sort(PETURBED_PARAMS);
    to_remove = [];
    for i = 1:size(PETURBED_PARAMS,1)-1
        if PETURBED_PARAMS(i) == PETURBED_PARAMS(i+1)
            to_remove = [to_remove i];
        end
    end
     PETURBED_PARAMS(to_remove) = [];  
end
%look for any state peturbations required
STATE_PET = [];
if exist(strcat(modelname, '.stpt'),'file') == 2
    %file is a matrix with one row for each state to be perturbed.
    %col 1 is state number, col2 is time on, col3 is time off
    %col 4 is size of peturbation, col5 is what this means. a - absolute
    %size, f - a factor to multiply existing value by, s - sum with
    %existing value
    [state_no t_on pet_size pet_type] = textread(strcat(modelname, '.stpt'),'%d %f %f %c');
    for i = 1:size(state_no,1)
        STATE_PET(i).state = state_no(i);
        STATE_PET(i).time = t_on(i);
        STATE_PET(i).size = pet_size(i);
        STATE_PET(i).type = pet_type(i);
    end
end
%calculate seperate runs required if there are peturbations to states

if ~isempty(STATE_PET);
    if STATE_PET(1).time > 0
        timespans = [0 STATE_PET(1).time];
    else
        timespans = [];
    end
    for i = 1:size(STATE_PET,2)-1
        if STATE_PET(i).time ~= STATE_PET(i + 1).time
            timespans = [timespans; STATE_PET(i).time STATE_PET(i + 1).time];
        end
    end
    if STATE_PET(end).time < tmax
        timespans = [timespans;  STATE_PET(end).time tmax];
    end
else
   timespans = [0 tmax];     
end

numOps = size(OPREQUIRED,1);
%one column for each parameter to be output
OUTPUT = zeros(1,numOps);
%Estimate number of rows depending on length of simulation
numRows = fix(tmax * PointsPerDay / 24);
RESULTS = zeros(numRows, numOps);      %will contain time series of outputs. Made by appending OUTPUT
TIME_POINTS = zeros(numRows,1);    %col vector of integration times
if HAS_LD > 0            %matrix of light or dark, one col for each colour, one row for each timepoint
    LD_CYCLE = zeros(numRows, HAS_LD); 
end
if HAS_TEMP > 0
    TEMP_CYCLE = zeros(numRows,1); 
end
if ~isempty(PETURBED_PARAMS)
    PARAMETERS = zeros(numRows, size(PETURBED_PARAMS,1));
end

if strcmp(ODESolver, 'bvp4c')
    options = bvpset('BCJacobian', @bcjac, 'NMax', NMax);
else
    options = odeset('OutputFcn',@WriteData ,'Refine',refine, 'MaxStep', maxStep, 'RelTol', relt, 'AbsTol', abst);  %WriteData called after every successful integration step
end
ubound = runSolver(ODESolver, options, timespans, State_Vals);

%get rid of any rows not needed if initialisation was an over-estimate of the numbe rof integration
% steps required
%remove excess points not needed
if size(RESULTS,1) > ubound
    RESULTS = RESULTS(1:ubound,:);
end
if size(LD_CYCLE,1) > ubound
    LD_CYCLE = LD_CYCLE(1:ubound,:);
end
if  size(TEMP_CYCLE,1) > ubound
    TEMP_CYCLE = TEMP_CYCLE(1:ubound,:);
end
if size(TIME_POINTS,1) > ubound
    TIME_POINTS = TIME_POINTS(1:ubound,:);
end
if size(PARAMETERS,1) > ubound
    PARAMETERS = PARAMETERS(1:ubound,:);
end

%write results to file
%Time values
fid = fopen('Time.txt', 'wt');
fprintf(fid, '%f\n', TIME_POINTS);
status = fclose(fid);
%results
fid = fopen('Data.txt', 'wt');
for row = 1:size(RESULTS,1)
     for col = 1:size(RESULTS, 2)
         fprintf(fid, '%f	', RESULTS(row, col));
     end
     fprintf(fid, '\n');
end
status = fclose(fid);

if HAS_LD > 0
    fid = fopen('LD.txt', 'wt');
    for row = 1:size(LD_CYCLE,1)
        for col = 1:size(LD_CYCLE, 2)
            fprintf(fid, '%f	', LD_CYCLE(row, col));
        end
        fprintf(fid, '\n');
    end
    status = fclose(fid);
end
if HAS_TEMP > 0
   fid = fopen('Temp.txt', 'wt');
   fprintf(fid, '%f\n', TEMP_CYCLE);
   status = fclose(fid);
end
if HAS_VERN > 0
    fid = fopen('Vern.txt', 'wt');
    for row = 1:size(VERN_VALS, 2)
        fprintf(fid, '%s', VERN_VALS{row});
    end
    status = fclose(fid);
end
if ~isempty(PETURBED_PARAMS)
    fid = fopen('ParamVals.txt', 'wt');
    fprintf(fid, '%f	', PETURBED_PARAMS');
    fprintf(fid, '\n');
    for row = 1:size(PARAMETERS,1)
        for col = 1:size(PARAMETERS, 2)
            fprintf(fid, '%f	', PARAMETERS(row, col));
        end
        fprintf(fid, '\n');
    end
    status = fclose(fid);
end

if ~ isempty(ENVFILE) 
    DoCalculations(modelname);
end

%plot(TIME_POINTS, RESULTS(:, [1 2 3]));
%plot(time, results(:, [1 2 3]));
%plot(TIME_POINTS, TEMP_CYCLE(:,1));

%-------------------------------------------------------------------------
function y = state_t(yt, t)
%Modifies one of the model states of the values according to any
%peturbations required
global STATE_PET;

y = yt;
for i = 1:size(STATE_PET,2)
    if STATE_PET(i).time == t
        if STATE_PET(i).type == 'a'
            y(STATE_PET(i).state) = STATE_PET(i).size;
        elseif STATE_PET(i).type == 'f'
            y(STATE_PET(i).state) = y(STATE_PET(i).state) * STATE_PET(i).size;
        else
            y(STATE_PET(i).state) = y(STATE_PET(i).state) + STATE_PET(i).size;
        end
        if y(STATE_PET(i).state) < 0 
            y(STATE_PET(i).state) = 0;
        end
    end
end
%--------------------------------------------------------------------------
%Index values for PARAM_VALS depend .prm file in interface, and
%must match those in this function

function P = param_t(t)
%Modifies one of the model paramters of the values according to any
%peturbations required
global PARAM_PET;
global PARAM_VALS;
    
P = PARAM_VALS;
for i = 1:size(PARAM_PET,2)
    if (t >= PARAM_PET(i).on) & ((t < PARAM_PET(i).off) | (PARAM_PET(i).on > PARAM_PET(i).off))
        if PARAM_PET(i).type == 'a'
            P(PARAM_PET(i).param) = PARAM_PET(i).size;
        elseif PARAM_PET(i).type == 'f'
            P(PARAM_PET(i).param) = P(PARAM_PET(i).param) * PARAM_PET(i).size;
        else
            P(PARAM_PET(i).param) = P(PARAM_PET(i).param) + PARAM_PET(i).size;
        end
    end
end
%--------------------------------------------------------------------------
function status = WriteData(t, y, flag)

global OUTPUT;
global RESULTS;
global TIME_POINTS;
global HAS_LD;
global LD_CYCLE;
global HAS_TEMP;
global TEMP_CYCLE;
global LD_VAL;
global TEMP_VAL;
global PETURBED_PARAMS;
global P;   %param values at time t
global PARAMETERS;  
persistent NextRow;
persistent i;

%This function is called after every valid integration step
if strcmp(flag, '') ~= 0     %if not initial or final step, but a valid integation step, append RESULTS  
   NextRow = NextRow + 1;
   RESULTS(NextRow, :) = OUTPUT;
   TIME_POINTS(NextRow) = t;
   if HAS_LD > 0
       LD_CYCLE(NextRow,:) = LD_VAL;
   end
   if HAS_TEMP > 0
       TEMP_CYCLE(NextRow,:) = TEMP_VAL;
   end
   if ~isempty(PETURBED_PARAMS)
        PARAMETERS(NextRow,:) = P(PETURBED_PARAMS)';
    end
elseif strcmp(flag, 'init')
   if isempty(i)
       i = 1;
       NextRow = 1;   %first call
       RESULTS(NextRow, :) = OUTPUT;
       TIME_POINTS(NextRow) = 0;
       if HAS_LD > 0
            LD_CYCLE(NextRow,:) = LD_VAL;
       end
       if HAS_TEMP > 0
            TEMP_CYCLE(NextRow,:) = TEMP_VAL;
       end
       if ~isempty(PETURBED_PARAMS)
            PARAMETERS(NextRow,:) = P(PETURBED_PARAMS)';
        end
   end
end
status = 0;
%-----------------------------------------------------------------------
function val = TEMP(t)

global HAS_TEMP;
global TEMP_DEF;
persistent idx;

if isempty(idx)
    idx = 1;
end

if HAS_TEMP > 0
    if TEMP_DEF(idx,1) < t %time has moved forward since last call
        %find last set of values before current time
        while (idx < size(TEMP_DEF,1)) & (TEMP_DEF(idx + 1, 1) <= t) 
            idx = idx + 1;
        end
    elseif TEMP_DEF(idx,1) > t  %time has moved back
        while (idx > 1) & (TEMP_DEF(idx, 1) > t) 
            idx = idx - 1;
        end
    end 
    val = TEMP_DEF(idx,2);
else
    val = [];
end
%--------------------------------------------------------------------------
function val = LD(t)
%calculates light  or dark  at a time t for an environmental regime
%retruns a row vector of values, one element for each colour
global LD_DEF;
global HAS_LD;
persistent idx;

if isempty(idx)
    idx = 1;
end

if HAS_LD > 0
    if LD_DEF(idx,1) < t %time has moved forward since last call
        %find last set of values before current time
        while (idx < size(LD_DEF,1)) & (LD_DEF(idx + 1, 1) <= t) 
            idx = idx + 1;
        end
    elseif LD_DEF(idx,1) > t  %time has moved back
        while (idx > 1) & (LD_DEF(idx, 1) > t) 
            idx = idx - 1;
        end
    end 
    val = LD_DEF(idx, 2:end);  
else
    val = [];   %model has no LD cycle
end
%--------------------------------------------------------------------------
function val = runSolver(ODESolver, options, timespans, StateValues)

global OPREQUIRED;
global TIME_POINTS;
global RESULTS;

val = 0;
for i = 1:size(timespans,1)  %one run for each state peturbation
    %add peturbations
    StateValues = state_t(StateValues, timespans(i,1)); %apply peturbation
    switch char(ODESolver)
        case 'ode45'
            [t, r] = ode45(@equations, timespans(i,:), StateValues, options);
        case 'ode23'
            [t, r] = ode23(@equations, timespans(i,:), StateValues, options);
        case 'ode113'
            [t, r] = ode113(@equations, timespans(i,:), StateValues, options);
        case 'ode23s'
            [t, r] = ode23s(@equations, timespans(i,:), StateValues, options);
        case 'ode23t'
            [t, r] = ode23t(@equations, timespans(i,:), StateValues, options);
        case 'ode23tb'
            [t, r] = ode23tb(@equations, timespans(i,:), StateValues, options);
        case 'ode15s'
            [t, r] = ode15s(@equations, timespans(i,:), StateValues, options);
        case 'bvp4c'
            % Searching for the limit cycle
            %read results from previous run
            fid = fopen('Time.txt', 'rt');
            t = fscanf(fid, '%f');
            status = fclose(fid);
            delete('Time.txt');

            fid = fopen('Data.txt', 'rt');
            y = [];
            while ~feof(fid)
                row = fscanf(fid, '%f', size(OPREQUIRED,1));
                y = [y; row'];
            end
            status = fclose(fid);
            delete('Data.txt');
            rinit.x = t;   %rinit represents a 24 hour time series
            rinit.y = y';  %note transpose
            lastwarn('');
            try                 
                lc = bvp4c(@equations, @bc, rinit, options);
                [msgstr msgid] = lastwarn;
                if (~strcmp(msgid, ''))
                    ReturnMsg('BVS does not converge! Using seed instead.');
                    lc = rinit;
                else
                    ReturnMsg('Limit cycle found successfully!');
                end
            catch
                ReturnMsg('Limit cycle not found! Using seed instead.');
                lc = rinit;
            end
            TIME_POINTS = lc.x;
            RESULTS = lc.y';    %note transpose
            t = TIME_POINTS;
            r = RESULTS;
    end     
    
    StateValues = r(end,:); %use final states as initial for next section
    if i > 1
       val = val - 1;
    end
    val = val + size(t,1);
end

%--------------------------------------------------------------------------
function [dBCdya, dBCdyb] = bcjac (ya, yb)

global OPREQUIRED
% flowerin4bcjac provides jacobian of BCs.

dBCdya = eye(size(OPREQUIRED,1));       % ya(i) only appears in the i-th eqn. 
dBCdyb = - eye(size(OPREQUIRED,1));     % -yb(i) only appears in the i-th eqn.

%-------------------------------------------------------------------------
function f = bc(ya, yb)

f = ya - yb;    %state y must have same value at time a and time b, ie a limit cycle
%a is start, b is end

%-------------------------------------------------------------------------
function ReturnMsg(s)
%write an exiting message tofile for calling application to read
fid = fopen('ReturnMsg.txt', 'wt');
fprintf(fid,'%s', s);
status = fclose(fid);

%-------------------------------------------------------------------------
function dydt = equations(t, y)
%P contains values for list of parameters
%in the order in which they appear in file modelname.prm. Must be referred to using these index values in this file

global OUTPUT;
global LD_VAL;
global LD_DEF;
global TEMP_VAL;
global OPREQUIRED;
global VERN_VALS;
global P;   %writedata must also see this in case of peturbations.

LD_VAL = LD(t);
TEMP_VAL = TEMP(t);


%apply a peturbation to one of the model params if in correct time period
P = param_t(t); %parameter values at time t

%TO DO : INSERT EQUATIONS HERE

Dark = 1 - LD_VAL(1)
if Dark < 0
	Dark = 0;
end
dydt = zeros(21,1);
% cL(m) - Level of LHY mRNA
dydt(1) = (P(3)^P(4)) / (P(3)^P(4) + y(16)^P(4)) * (LD_VAL(1) * (P(1)*y(13) + P(2)) + (P(5) * y(9)^P(6)) / (P(7)^P(6) + y(9)^P(6)))  -(P(8) * y(1)) / (P(9) + y(1));
% cL(c) - Level of cytoPlasmic LHY Protein
dydt(2) = P(79) * y(17) + P(10)*y(1) - P(11) * y(2) + P(12) * y(3) - (P(13) * y(2)) / (P(14) + y(2));
% cL(n) - Level of nuclear LHY Protein
dydt(3) = P(11)*y(2) - P(12)*y(3) - (P(15)*y(3)) / (P(16) + y(3));
% cT(m) - Level of TOC1 mRNA
dydt(4) = ((P(17) * y(12)^P(18)) / (P(19)^P(18) + y(12)^P(18))) * ((P(20)^P(21)) / (P(20)^P(21) + y(3)^P(21))) - (P(22) * y(4)) / (P(23) + y(4));
% cT(c) - Level of cytoPlasmic TOC1 Protein
dydt(5) = P(81) * y(18) + P(24) * y(4) - P(25)*y(5) + P(26) * y(6) - (Dark * P(27) + P(28)) * (y(5)) / (P(29) + y(5));
% cT(n) - Level of nuclear TOC1 Protein
dydt(6) = P(25) * y(5) - P(26) * y(6) - (Dark * P(30) + P(31)) * (y(6)) / (P(32) + y(6));
% cX(m) - Level of X mRNA
dydt(7) = (P(33) * y(6)^P(34)) / (P(35)^P(34) + y(6)^P(34)) - (P(36) * y(7)) / (P(37) + y(7));
% cX(c) - Level of cytoPlasmic Protein X
dydt(8) = P(83) * y(19) + P(38) * y(7) - P(39) * y(8) + P(40) * y(9) - (P(41) * y(8))/(P(42) + y(8));
% cX(n) - Level of nuclear Protein X
dydt(9) = P(39) * y(8) - P(40) * y(9) - (P(43) * y(9)) / (P(44) + y(9));
% cY(m) - Level of Y mRNA
dydt(10) = (LD_VAL(1) * P(45) * y(13) + ((LD_VAL(1) * P(46) + P(47)) * P(48)^P(50)) / (P(48)^P(50) + y(6)^P(50))) * ((P(49)^P(51)) / (P(49)^P(51) + y(3)^P(51))) - (P(52) * y(10)) / (P(53) + y(10));
% cY(c) - Level of cytoplasmic protein Y
dydt(11) = P(85) * y(20) + P(54) * y(10) - P(55) * y(11) + P(56) * y(12) - (P(57) * y(11)) / (P(58) + y(11));
% cY(n) - Level of nuclear protein Y
dydt(12) = P(55) * y(11) - P(56) * y(12) - (P(59) * y(12)) / (P(60) + y(12));
% cP(n) - Level of nuclear protein P
dydt(13) = Dark * P(61) - (P(63) * y(13)) / (P(62) + y(13)) - P(64) * LD_VAL(1) * y(13);
% cA(m) - Level of APRR7/9 mRNA
dydt(14) = LD_VAL(1) * P(65) * y(13) + (P(67) * y(3)^P(66)) / (P(68)^P(66) + y(3)^P(66)) - (P(69) * y(14)) / (P(70) + y(14));
% cA(c) - Level of cytoplasmic APRR7/9
dydt(15) = P(88) * y(21) + P(71)*y(14) - P(72) * y(15)  + P(73) * y(16) - (P(74) * y(15)) / (P(75) + y(15));
% cA(n) - Level of nuclear APRR7/9
dydt(16) = P(72) * y(15) - P(73) * y(16) - (P(76) * y(16)) / (P(77) + y(16));
% cL(m)ox - Level of LHY mRNA from constitutive gene
dydt(17) = P(78)-(P(8) * y(17)) / (P(9) + y(17));
% cT(m)ox - Level of TOC1 mRNA from constitutive gene
dydt(18) = P(80)-(P(22) * y(18)) / (P(23) + y(18));
% cX(m)ox - Level of X mRNA from constitutive gene
dydt(19) = P(82)- (P(36) * y(19)) / (P(37) + y(19));
% cY(m)ox - Level of Y mRNA from constitutive gene
dydt(20) = P(84)- (P(52) * y(20)) / (P(53) + y(20));
% cA(m)ox - Level of APRR7/9 mRNA from constitutive gene
dydt(21) = P(87)-(P(69) * y(21)) / (P(70) + y(21));

%Model outputs

for i = 1:size(OPREQUIRED,1)
	switch OPREQUIRED(i,1)
		%TO DO : INSERT OUTPUTS HERE
		case 1
			OUTPUT(i) = y(1); % cL(m) - Level of LHY mRNA - nM
		case 2
			OUTPUT(i) = y(2); % cL(c) - Level of cytoplasmic LHY protein - nM
		case 3
			OUTPUT(i) = y(3); % cL(n) - Level of nuclear LHY protein - nM
		case 4
			OUTPUT(i) = y(4); % cT(m) - Level of TOC1 mRNA - nM
		case 5
			OUTPUT(i) = y(5); % cT(c) - Level of cytoplasmic TOC1 protein - nM
		case 6
			OUTPUT(i) = y(6); % cT(n) - Level of nuclear TOC1 protein - nM
		case 7
			OUTPUT(i) = y(7); % cX(m) - Level of X mRNA - nM
		case 8
			OUTPUT(i) = y(8); % cX(c) - Level of cytoplasmic protein X - nM
		case 9
			OUTPUT(i) = y(9); % cX(n) - Level of nuclear protein X - nM
		case 10
			OUTPUT(i) = y(10); % cY(m) - Level of Y mRNA - nM
		case 11
			OUTPUT(i) = y(11); % cY(c) - Level of cytoplasmic protein Y - nM
		case 12
			OUTPUT(i) = y(12); % cY(n) - Level of nuclear protein Y - nM
		case 13
			OUTPUT(i) = y(13); % cP(n) - Level of nuclear protein P - nM
		case 14
			OUTPUT(i) = y(14); % cA(m) - Level of APP7/9 mRNA - nM
		case 15
			OUTPUT(i) = y(15); % cA(c) - Level of cytoplasmic APRR7/9 protein - nM
		case 16
			OUTPUT(i) = y(16); % cA(n) - Level of nuclear APRR7/9 protein - nM
		case 17
			OUTPUT(i) = ((P(3)^P(4)) / (P(3)^P(4) + y(16)^P(4))) * (LD_VAL(1) * (P(1)*y(13) + P(2))); % cL(m)LTsc - Rate of light-dependent LHY transcription - nM/h
		case 18
			OUTPUT(i) = ((P(3)^P(4)) / (P(3)^P(4) + y(16)^P(4))) * ((P(5) * y(9)^P(6)) / (P(7)^P(6) + y(9)^P(6))); % cL(m)Tsc - Rate of light-independent LHY transcription - nM/h
		case 19
			OUTPUT(i) = (P(8) * y(1)) / (P(9) + y(1)); % cL(m)Dgr - Rate of LHY mRNA degradation - nM/h
		case 20
			OUTPUT(i) = P(10)*y(1); % cL(m)Tsl - Rate of native LHY translation - nM/h
		case 21
			OUTPUT(i) = (P(13) * y(2)) / (P(14) + y(2)); % cL(c)Dgr - Rate of cytoplasmic LHY protein degradation - nM/h
		case 22
			OUTPUT(i) = P(11) * y(2) - P(12) * y(3); % cL(n)Prd - Net rate of LHY protein movement into the nucleus - nM/h
		case 23
			OUTPUT(i) = (P(15)*y(3)) / (P(16) + y(3)); % cL(n)Dgr - Rate of nuclear LHY protein degradation - nM/h
		case 24
			OUTPUT(i) = ((P(17) * y(12)^P(18)) / (P(19)^P(18) + y(12)^P(18))) * ((P(20)^P(21)) / (P(20)^P(21) + y(3)^P(21))); % cT(m)Tsc - Rate of TOC1 transcription - nM/h
		case 25
			OUTPUT(i) = (P(22) * y(4)) / (P(23) + y(4)); % cT(m)Dgr - Rate of TOC1 mRNA degradation - nM/h
		case 26
			OUTPUT(i) = P(24) * y(4); % cT(m)Tsl - Rate of native TOC1 translation - nM/h
		case 27
			OUTPUT(i) = (Dark * P(27)) * (y(5)) / (P(29) + y(5)); % cT(c)LDgr - Rate of light-dependent cytoplasmic TOC1 protein degradation - nM/h
		case 28
			OUTPUT(i) = P(28) * (y(5)) / (P(29) + y(5)); % cT(c)Dgr - Rate of light-independent cytoplasmic TOC1 protein degradation - nM/h
		case 29
			OUTPUT(i) = P(25) * y(5) - P(26) * y(6); % cT(n)Prd - Net rate of TOC1 protein movement into the nucleus - nM/h
		case 30
			OUTPUT(i) = (Dark * P(30)) * (y(6)) / (P(32) + y(6)); % cT(n)LDgr - Rate of light-dependent nuclear TOC1 protein degradation - nM/h
		case 31
			OUTPUT(i) = P(31) * (y(6)) / (P(32) + y(6)); % cT(n)Dgr - Rate of light-independent nuclear TOC1 protein degradation - nM/h
		case 32
			OUTPUT(i) = (P(33) * y(6)^P(34)) / (P(35)^P(34) + y(6)^P(34)); % cX(m)Tsc - Rate of X transcription - nM/h
		case 33
			OUTPUT(i) = (P(36) * y(7)) / (P(37) + y(7)); % cX(m)Dgr - Rate of X mRNA degradation - nM/h
		case 34
			OUTPUT(i) = P(38) * y(7); % cX(m)Tsl - Rate of native protein X translation - nM/h
		case 35
			OUTPUT(i) = (P(41) * y(8))/(P(42) + y(8)); % cX(c)Dgr - Rate of cytoplasmic protein X degradation - nM/h
		case 36
			OUTPUT(i) = P(39) * y(8) - P(40) * y(9); % cX(n)Prd - Net rate of protein X movement into the nucleus - nM/h
		case 37
			OUTPUT(i) = (P(43) * y(9)) / (P(44) + y(9)); % cX(n)Dgr - Rate of nuclear protein X degradation - nM/h
		case 38
			OUTPUT(i) = (LD_VAL(1) * P(45) * y(13) + ((LD_VAL(1) * P(46) * P(48)^P(50)) / (P(48)^P(50) + y(6)^P(50)))) * ((P(49)^P(51)) / (P(49)^P(51) + y(3)^P(51))); % cY(m)LTsc - Rate of light-dependent Y transcription - nM/h
		case 39
			OUTPUT(i) = ((P(47) * P(48)^P(50)) / (P(48)^P(50) + y(6)^P(50))) * ((P(49)^P(51)) / (P(49)^P(51) + y(3)^P(51))); % cY(m)Tsc - Rate of light-independent Y transcription - nM/h
		case 40
			OUTPUT(i) = (P(52) * y(10)) / (P(53) + y(10)); % cY(m)Dgr - Rate of Y mRNA degradation - nM/h
		case 41
			OUTPUT(i) = P(54) * y(10); % cY(m)Tsl - Rate of native protein Y translation - nM/h
		case 42
			OUTPUT(i) = (P(57) * y(11)) / (P(58) + y(11)); % cY(c)Dgr - Rate of cytoplasmic protein Y degradation - nM/h
		case 43
			OUTPUT(i) = P(55) * y(11) - P(56) * y(12); % cY(n)Prd - Net rate of protein Y movement into the nucleus - nM/h
		case 44
			OUTPUT(i) = (P(59) * y(12)) / (P(60) + y(12)); % cY(n)Dgr - Rate of nuclear protein Y degradation - nM/h
		case 45
			OUTPUT(i) = Dark * P(61); % cP(n)Prd - Rate of native nuclear protein P production - nM/h
		case 46
			OUTPUT(i) = P(64) * LD_VAL(1) * y(13); % cP(n)LDgr - Rate of light-dependent nuclear protein P degradation - nM/h
		case 47
			OUTPUT(i) = (P(63) * y(13)) / (P(62) + y(13)); % cP(n)Dgr - Rate of light-independent nuclear protein P degradation - nM/h
		case 48
			OUTPUT(i) = LD_VAL(1) * P(65) * y(13); % cA(m)LTsc - Rate of light-dependent APRR7/9 transcription - nM/h
		case 49
			OUTPUT(i) = (P(67) * y(3)^P(66)) / (P(68)^P(66) + y(3)^P(66)); % cA(m)Tsc - Rate of light-independent APRR7/9 transcription - nM/h
		case 50
			OUTPUT(i) = (P(69) * y(14)) / (P(70) + y(14)); % cA(m)Dgr - Rate of APRR7/9 mRNA degradation - nM/h
		case 51
			OUTPUT(i) = P(71)*y(14); % cA(m)Tsl - Rate of native APRR7/9 translation - nM/h
		case 52
			OUTPUT(i) = (P(74) * y(15)) / (P(75) + y(15)); % cA(c)Dgr - Rate of cytoplasmic APRR7/9 degradation - nM/h
		case 53
			OUTPUT(i) = P(72) * y(15) - P(73) * y(16); % cA(n)Prd - Net rate of APRR7/9 movement into the nucleus - nM/h
		case 54
			OUTPUT(i) = (P(76) * y(16)) / (P(77) + y(16)); % cA(n)Dgr - Rate of nuclear APRR7/9 degradation - nM/h
		case 55
			OUTPUT(i) = y(17); % cL(m)ox - Level of LHY mRNA from constitutive gene - nM
		case 56
			OUTPUT(i) = P(78); % cL(m)oxTsc - Rate of transcription of constitutive LHY gene - nM/h
		case 57
			OUTPUT(i) = (P(8) * y(17)) / (P(9) + y(17)); % cL(m)oxDgr - Rate of degradation of LHY mRNA from constitutive gene - nM/h
		case 58
			OUTPUT(i) = P(79) * y(17); % cL(m)oxTsl - Rate of translation of LHY mRNA from constitutive gene - nM/h
		case 59
			OUTPUT(i) = y(18); % cT(m)ox - Level of TOC1 mRNA from constitutive gene - nM
		case 60
			OUTPUT(i) = P(80); % cT(m)oxTsc - Rate of transcription of constitutive TOC1 gene - nM/h
		case 61
			OUTPUT(i) = (P(22) * y(18)) / (P(23) + y(18)); % cT(m)oxDgr - Rate of degradation of TOC1 mRNA from constitutive gene - nM/h
		case 62
			OUTPUT(i) = P(81) * y(18); % cT(m)oxTsl - Rate of translation of TOC1 mRNA from constitutive gene - nM/h
		case 63
			OUTPUT(i) = y(19); % cX(m)ox - Level of X mRNA from constitutive gene - nM
		case 64
			OUTPUT(i) = P(82); % cX(m)oxTsc - Rate of transcription of constitutive X gene - nM/h
		case 65
			OUTPUT(i) = (P(36) * y(19)) / (P(37) + y(19)); % cX(m)oxDgr - Rate of degradation of X mRNA from constitutive gene - nM/h
		case 66
			OUTPUT(i) = P(83) * y(19); % cX(m)oxTsl - Rate of translation of X mRNA from constitutive gene - nM/h
		case 67
			OUTPUT(i) = y(20); % cY(m)ox - Level of Y mRNA from constitutive gene - nM
		case 68
			OUTPUT(i) = P(84); % cY(m)oxTsc - Rate of transcription of constitutive Y gene - nM/h
		case 69
			OUTPUT(i) = (P(52) * y(20)) / (P(53) + y(20)); % cY(m)oxDgr - Rate of degradation of Y mRNA from constitutive gene - nM/h
		case 70
			OUTPUT(i) = P(85) * y(20); % cY(m)oxTsl - Rate of translation of Y mRNA from constitutive gene - nM/h
		case 71
			OUTPUT(i) = P(86); % cP(n)oxTsl - Rate of translation of Protein P from constitutive gene - nM/h
		case 72
			OUTPUT(i) = y(21); % cA(m)ox - Level of APRR7/9 mRNA from constitutive gene - nM
		case 73
			OUTPUT(i) = P(87); % cA(m)oxTsc - Rate of transcription of constitutive APRR7/9 gene - nM/h
		case 74
			OUTPUT(i) = (P(69) * y(21)) / (P(70) + y(21)); % cA(m)oxDgr - Rate of degradation of APRR7/9 mRNA from constitutive gene - nM/h
		case 75
			OUTPUT(i) = P(88) * y(21); % cA(m)oxTsl - Rate of translation of APRR7/9 mRNA from constitutive gene - nM/h
	end
end
%======================================================================
function DoCalculations(modelname)  
%   TO DO : Insert model specific code in here


%====================================================================
function final(modelname);
%   TO DO : Insert model specific code in here

%parameter has default value. If name present, eg 'name=' but no value,
%then it will be assigned zero from array 'values'

